import os
import datetime
import offer
import re
import uuid
import base64
import gluon
import clientsupport

MIN_PRICE = 1
MIN_QTY = 1
MAX_PRICE = 1000000
MAX_QTY = 1000000

def user(): return dict(form=auth())
def download(): return response.download(request,db)
def call():
    session.forget()
    return service()
### end requires

@auth.requires_membership('User')
def index():
    form = SQLFORM(db.seetsoc_ohs_xmlhistory, fields = ['filename', 'previd'])

    if form.accepts(request.vars, session, dbio=False):
        if form.vars.filename == "":
            response.flash = 'No filename supplied, please try again.'
            return dict(uploadform = form)
        if (form.vars.previd == None):
            result = parse_uploaded_xml(os.path.join(request.folder, 'uploads', form.vars.filename_newfilename))
        else:
            result = parse_uploaded_xml(os.path.join(request.folder, 'uploads', form.vars.filename_newfilename), previd = form.vars.previd)
        response.flash = result[1]
    elif form.errors:
        response.flash = 'Invalid filename supplied, please try again.'
    else:
        response.flash = 'Click "Browse" and select the file to upload.'

    return dict(uploadform=form)

def register_bad_response(ack_time, eic, xml_file,):
    db.seetsoc_ohs_xmlhistory.insert(offer_id = None,
                                     ack_time = ack_time,
                                     eic = eic,
                                     accepted = False,
                                     filename = xml_file,
                                     valid = True,
                                     entered_transaction = False)

def parse_uploaded_xml(xml_file, previd = None, force_user = None):
    if force_user == None:
        crt_user = db.auth_user[auth.user_id]
    else:
        crt_user = db.auth_user[1]

    o = None
    validity = None
    try:
        o = offer.Offer(xml_file, None)
    except Exception as detail:
        return [False, detail]

    validity = o.is_valid(datetime.datetime.now(), MAX_QTY, MIN_QTY,
                          MAX_PRICE, MIN_PRICE)

    # Ensure that the user is not placing an offer for an interval on
    # which he has already done so.

    previd_query = (db.seetsoc_ohs_bids.offer_id ==  db.seetsoc_ohs_offer.id and db.seetsoc_ohs_offer.delivery_date == o.get_delivery_date())
    rows = db(previd_query).select(db.seetsoc_ohs_bids.interval)

    for row in rows:
        for entry in o.energy_offer:
            if (entry[0] == row['seetsoc_ohs_bids.interval']):
                register_bad_response(datetime.datetime.now(), crt_user.TSO, xml_file)
                return [False, "You have already placed a bid for interval " + str(row['seetsoc_ohs_bids.interval']) + ". If you want to update a bid for that interval, submit a new version of the offer it was placed in."]

    # Additional verifications for the old ID:
    # - Does the old ID of the offer exist?
    if previd != None:
        previd_query = db.seetsoc_ohs_offer.id == previd
        previd_set = db(previd_query)
        rows = previd_set.select('id')
        if len(rows.records) == 0:
            register_bad_response(datetime.datetime.now(), crt_user.TSO, xml_file)
            return [False, "An offer with the previous ID you entered could not be found"];
    # - Is it an offer for the same user?
        prevoff_query = db.seetsoc_ohs_offer.id == previd
        prevoff_set = db(prevoff_query)
        rows = prevoff_set.select('eic')
        if (int(rows.first()['eic'])) != crt_user.TSO:
            register_bad_response(datetime.datetime.now(), crt_user.TSO, xml_file)
            return [False, "An offer placed by you with the previous offer ID you entered could not be found"];

    if (validity[0] == True):
        offer_id = commit_offer_to_db(xml_file, o, crt_user, previd)
        return [True, "The offer was accepted and registered with ID number " + str(offer_id)]
    else:
        db.seetsoc_ohs_xmlhistory.insert(offer_id = None,
                                     ack_time = datetime.datetime.now(),
                                     eic = crt_user.TSO,
                                     accepted = False,
                                     filename = xml_file,
                                     valid = True,
                                     entered_transaction = False)
        return [False, validity[1][0]]

def commit_offer_to_db(xml_file, off, crt_user, old_id = None):
    # Insert the offer in the offers table
    ohs_offer_id = db.seetsoc_ohs_offer.insert(eic = crt_user.TSO,
                                               delivery_date = off.get_delivery_date(),
                                               creation_date = off.get_creation_date(),
                                               msg_version = off.message_version,
                                               off_type = off.offer_type)
    # Insert the bids in the bids table
    bids = []
    for entry in off.energy_offer:
        bids.append(dict(offer_id = ohs_offer_id,
                         interval = entry[0],
                         price = entry[1],
                         qty = entry[2],
                         eng_type = off.get_energy_type(),
                         bid_type = off.offer_type))
    db.seetsoc_ohs_bids.bulk_insert(bids)

    # Insert the XML file in the XML history file
    db.seetsoc_ohs_xmlhistory.insert(offer_id = ohs_offer_id,
                                     ack_time = datetime.datetime.now(),
                                     eic = crt_user.TSO,
                                     accepted = True,
                                     filename = xml_file,
                                     valid = True,
                                     entered_transaction = False)

    db.commit()

    # Remove old version from the database
    if old_id != None:
        db(db.seetsoc_ohs_bids.offer_id == old_id).delete()
        db(db.seetsoc_ohs_offer.id == old_id).delete()

    db.commit()

    return ohs_offer_id

def encode_filename(filename, tablename, fieldname):
    filename = os.path.basename(filename or file.filename)
    m = re.compile('\.(?P<e>\w{1,5})$').search(filename)
    extension = m and m.group('e') or 'xml'
    uuid_key = str(uuid.uuid4()).replace('-', '')[-16:]
    encoded_filename = base64.b16encode(filename).lower()
    newfilename = '%s.%s.%s.%s' % \
        (tablename, fieldname, uuid_key, encoded_filename)
    newfilename = newfilename[:122]+'.'+extension
    return newfilename

@service.soap('remote_upload', returns={'message':str}, args={'offer_stream':str, 'previd':int})
def remote_upload(offer_stream, previd):
    # Obtain an unique name for the file and copy it to the upload folder
    # for reference.

    target_filename = encode_filename('remote_upload', 'seetsoc_ohs_xmlhistory', 'filename')
    target_path =  os.path.join(request.folder, 'uploads', target_filename)

    gluon.fileutils.write_file(target_path, offer_stream);

    if (previd == -1):
        previd = None

    result = parse_uploaded_xml(os.path.join(request.folder, 'uploads', target_filename),
                                previd, force_user = True)

    return result[1]

@service.soap('client_min_price', returns={'ans':int}, args={})
def client_min_price():
    return 10

@service.soap('client_max_price', returns={'ans':int}, args={})
def client_min_price():
    return 1000

@service.soap('client_min_qty', returns={'ans':int}, args={})
def client_min_qty():
    return 25

@service.soap('client_max_qty', returns={'ans':int}, args={})
def client_max_qty():
    return 3500

@service.soap('client_latest_version', returns={'version':int}, args={})
def client_latest_version():
    return clientsupport.client_latest_version()

