import merit

# Merit order tests
#
# Rules: there is never an equal number of sales and buy offers
#
# Test 1: Normal tests, in which the settling point is in the middle
# Test 2: The intersection point is at the first offer
# Test 3: The intersection point is at the last offer
# Test 4: The intersection point is at the first and only offer
# Test 5: There is no intersection point

# Test 1 (Note: to test sorting, the lists are *not* ordered as the table)
#
# Buying offers:                       Selling:
#  __________________________________   __________________________________
# | Offer ID |  Bid ID | Price | Qty | | Offer ID |  Bid ID | Price | Qty |
# |    1     |    1    | 100   | 10  | |    6     |    7    |  80   | 40  |
# |    1     |    2    | 110   | 15  | |    7     |    8    |  90   | 25  |
# |    2     |    3    | 130   | 20  | |    8     |    9    | 120   | 10  |
# |    3     |    4    | 140   | 20  | |    8     |    10   | 140   | 20  |
# |    4     |    5    | 150   | 40  | |    8     |    11   | 150   | 40  |
# |    5     |    6    | 155   | 30  | |          |         |       |     |
#  ----------------------------------   ----------------------------------
#
# Expected output: - the settlement is done at 125.0
#                  - Buying offers will be 1 and 2
#                  - Selling offers: both 6 and 7 would be valid but only 6
#                    will be selected (and incompletely) because it has to
#                    match the quantity.
#

# buy_offers =  [ [1, 1, 100, 10, 1, 10, 1, "TSO 1"],
#                 [2, 3, 130, 20, 1, 20, 2, "TSO 2"],
#                 [1, 2, 110, 15, 1, 15, 1, "TSO 1"],
#                 [5, 6, 155, 30, 1, 30, 2, "TSO 2"],
#                 [4, 4, 140, 40, 1, 40, 3, "TSO 3"],
#                 [3, 4, 130, 20, 1, 20, 1, "TSO 1"]]

# sale_offers = [ [8, 11, 150, 40, 1, 40, 4, "TSO 4"],
#                 [6, 7,  80,  40, 1, 40, 3, "TSO 3"],
#                 [8, 10, 140, 20, 1, 20, 4, "TSO 4"],
#                 [8, 9 , 120, 10, 1, 10, 4, "TSO 4"],
#                 [7, 8,  190, 25, 1, 25, 5, "TSO 5"]]

sale_offers =  [ [1, 1, 112, 10, 1, 10, 1, "TSO 1"],
                [2, 3, 115, 12, 1, 12, 2, "TSO 2"],
                [1, 2, 120, 22, 1, 22, 1, "TSO 1"],
                [5, 6, 126, 30, 1, 30, 2, "TSO 2"],
                [4, 4, 129, 23, 1, 32, 3, "TSO 3"],
                [3, 4, 130, 37, 1, 37, 1, "TSO 1"]]

buy_offers = [  [8, 11, 130, 33, 1, 33, 4, "TSO 4"],
                [6, 7,  125, 24, 1, 24, 3, "TSO 3"],
                [8, 10, 120, 24, 1, 24, 4, "TSO 4"],
                [8, 9 , 111, 21, 1, 21, 4, "TSO 4"],
                [7, 8,  100, 15, 1, 15, 5, "TSO 5"],
                [7, 12, 90, 21, 1, 21, 5, "TSO 5"]]

order = merit.mo_upward_tertiary(buy_offers, sale_offers)

print "Ok, output:"

merit.debug("mcp: " + str(order[2]))
merit.debug(order[0])
